///
// @file msa_config_http.h
// @brief header file for http setting of modem sub application
// @details
// @author This software is contributed or developed by KYOCERA Corporation.
// @copyright (C) 2022 KYOCERA Corporation
//
// SPDX-License-Identifier: MIT
//
///

// Define to prevent recursive inclusion -----------------------
#ifndef __MSA_CONFIG_HTTP_H__
#define __MSA_CONFIG_HTTP_H__

#ifdef C_SYNTAX
#include "basicInformation.h"
#endif // C_SYNTAX
#include "eventLib.h"
#include "logAPI.h"
#include "flashLib.h"
#include "bufferLib.h"
#include "cmd_str.h"
#include "modemMgrTask.h"
#include "sappModemAPI.h"

//////////////////////////////////////////////////////////////////
// sapp_modem.krsの利用方法 //
//////////////////////////////////////////////////////////////////
// 0.はじめに
//   サブアプリsapp_modem.krsはLTE送受信の機能を一括で担うため、
//   本サブアプリを利用することでユーザーアプリケーションとLTE送受信処理を独立させ
//   より自由度の高い機能実現が可能となる。
//   以下、sapp_modem.krsの利用方法を記載する。
// 
//   送信IDは1～8を用意しており、各接続先ごとに違うIDを割り振ることができる。
//   本ヘッダファイルではHTTP通信利用における設定ができる。
// 
// 1.プロトコルを利用する送信IDの選択
//   利用する送信IDの#defineを設定をする。非利用時は"undefを設定する。
#undef  MSA_CONFIG_01_USE_HTTP
#undef  MSA_CONFIG_02_USE_HTTP
#undef  MSA_CONFIG_03_USE_HTTP
#undef  MSA_CONFIG_04_USE_HTTP
#undef  MSA_CONFIG_05_USE_HTTP
#undef  MSA_CONFIG_06_USE_HTTP
#undef  MSA_CONFIG_07_USE_HTTP
#undef  MSA_CONFIG_08_USE_HTTP
// 
// 2.LTE通信で利用する送受信用のFIFOバッファや通信先や各種機能のパラメータ設定
//   本ファイルの80行目以降の「basic setting」「modem setting」に
//   #define設定を利用用途に合うよう設定する。
// 
//   各#define名は利用する送信IDの番号に合わせて、送信ID1を利用する場合は
//     MSA_CONFIG_01_API_TIMEOUT
//   送信ID2を利用する場合は
//     MSA_CONFIG_02_API_TIMEOUT
//   のように「_0X_」の部分の数字を正しく設定して利用すること。
// 
// 3.利用するアプリタスク(uapp_x.krs)へのモデム関連の処理記述
//   ここでは最もシンプルな送信処理のみを実行する場合の説明を行う。
//   コールバック処理を含む複雑な利用方法の説明は別マニュアルを参照。
// 
//   ＜送信IDの利用登録処理＞
//   アプリタスク(uapp_x.krs)の初期化処理event_init()の中で、以下APIを実行し
//   そのアプリで利用する送信IDを登録する。
//       MSA_SetLteSendId(MSA_SEND_ID_XX_LTE, MSA_SID_API_RECEIVE_DISABLE, 0, 0)
//   MSA_SEND_ID_XX_LTEのXXの部分には利用する送信IDを01や02の様に設定する。
//   本APIの実行成功(returnが0:EAPI_NO_ERROR)により、その送信IDの利用が可能となる。
// 
//   ＜LTEデータ送信の実行＞
//   アプリタスク(uapp_x.krs)の任意の関数内でFIFOバッファへのデータ書き込みを行うことで
//   LTE送信が可能。
//       rcplib_FIFOBUF_SetData(MSA_FIFO_BUF_XX_LTE_SEND, size, data);
//   MSA_FIFO_BUF_XX_LTE_SENDのXXの部分には利用する送信IDを01や02の様に設定する。
// 
//   実際の動作としては、sapp_modem.krsが1秒周期でFIFOバッファの状態を監視し
//   データが更新されていた場合にsapp_modem.krsから送信命令を行う動作となる。
// 
//   FIFOバッファの詳細な利用方法はbasicInformation.hまたはAPIリファレンス参照。
// 
//////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////
// basic setting (ユーザーの利用目的に合うように変更可能) //
//////////////////////////////////////////////////////////////////
#ifdef MSA_CONFIG_01_USE_HTTP
// <送信ID1の基本的な設定値>
#define MSA_CONFIG_01_API_TIMEOUT 30 * 1000          // 30秒のタイムアウトで通信関連のAPI実行を行う
#define MSA_CONFIG_01_SEND_ID     MSA_SEND_ID_01_LTE // 送信ID設定
#define MSA_CONFIG_01_SEND_BURST  1                  // 1送信処理あたりの連続送信数
#define MSA_CONFIG_01_USE_USB     MMG_NOTUSE_USB     // USB高速パス無効
#define MSA_CONFIG_01_DATA_RETAIN MMG_DATA_RETAIN    // 送信データ失敗時のretain機能有効
// <送信ID1の送信用FIFOバッファ設定値>
// 送信用FIFOバッファID
#define FIFO_BUF_LTE_SEND_01_ID MSA_FIFO_BUF_01_LTE_SEND
// 送信用FIFOバッファのデータサイズ
#define FIFO_BUF_LTE_SEND_01_DATA_SIZE 1024
// 送信用FIFOバッファの最大パケット数
#define FIFO_BUF_LTE_SEND_01_DEPTH_MAX 4
// 送信用FIFOバッファのオーバーライト設定
#define FIFO_BUF_LTE_SEND_01_OVERWRITE FIFOBUF_OVERWRITE_ENABLE
// <送信ID1の受信用FIFOバッファ設定値>
// 受信用FIFOバッファID
#define FIFO_BUF_LTE_RECV_01_ID MSA_FIFO_BUF_01_LTE_RECV
// 受信用FIFOバッファのデータサイズ
#define FIFO_BUF_LTE_RECV_01_DATA_SIZE 1024
// 受信用FIFOバッファの最大パケット数
#define FIFO_BUF_LTE_RECV_01_DEPTH_MAX 4
// 受信用FIFOバッファのオーバーライト設定
#define FIFO_BUF_LTE_RECV_01_OVERWRITE FIFOBUF_OVERWRITE_ENABLE
#endif // MSA_CONFIG_01_USE_HTTP

//////////////////////////////////////////////////////////////////
// modem setting (ユーザーの利用目的に合うように変更可能) //
//////////////////////////////////////////////////////////////////

#ifdef MSA_CONFIG_01_USE_HTTP
// <HTTP通信先設定値>
#define MSA_CONFIG_01_CONN_HOSTNAME      "uni.soracom.io"                  // 接続先ホスト名
#define MSA_CONFIG_01_CONN_PORT          80                                // 接続先ポート番号
#define MSA_CONFIG_01_HTTP_URL           "/"                               // URL
#define MSA_CONFIG_01_HTTP_HEADER1       "Content-Type: application/json"  // HTTPヘッダ1
#define MSA_CONFIG_01_HTTP_HEADER2       ""                                // HTTPヘッダ2
#define MSA_CONFIG_01_HTTP_HEADER3       ""                                // HTTPヘッダ3
#define MSA_CONFIG_01_HTTP_METHOD        MMG_METHOD_HTTPPOST               // HTTPメソッド(HTTP POSTを利用)
#endif // MSA_CONFIG_01_USE_HTTP

#endif
